<?php

    namespace common\models;

    use common\behaviors\MapsBehavior;
    use common\modules\comment\models\Comment;
    use common\modules\comment\models\Rating;
    use Yii;
    use yii\behaviors\BlameableBehavior;
    use yii\behaviors\TimestampBehavior;
    use yii\db\ActiveQuery;
    use yii\db\Expression;
    use yii\base\Component;

    /**
     * This is the model class for table "portfolio".
     * @property integer                   $portfolio_id
     * @property integer                   $user_id
     * @property string                    $name
     * @property string                    $link
     * @property string                    $date_add
     * @property integer                   $user_add_id Currently inactive attribute
     * @property integer                   $view_count
     * @property string                    $city
     * @property string                    $street
     * @property string                    $house
     * @property string                    $description
     * @property string                    $cover
     * @property integer                   $gallery_id
     * @property string                    $preview
     * @property PortfolioSpecialization[] $portfolioSpecializations
     * @property Specialization[]          $specializations
     * @property User                      $user
     * @property Comment[]                 $comment
     * @property Rating[]                  $rating
     * @property string                    $ratingValue
     * @method string minImg( string $dir, $width, $height = NULL ) Resizes image
     * @method Component watermark(string $path) Add watermark to image
     * @method string getWatermark(string $path, mixed $width, mixed $height) Get watermarked image
     * @method array ShowGallery( string $array ) Splits string to image paths array
     */
    class Portfolio extends \yii\db\ActiveRecord
    {

        /**
         * @inheritdoc
         */
        public static function tableName()
        {
            return 'portfolio';
        }

        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                [
                    'class'              => BlameableBehavior::className(),
                    'createdByAttribute' => 'user_id',
                    'updatedByAttribute' => false,
                ],
                [
                    'class'              => TimestampBehavior::className(),
                    'createdAtAttribute' => 'date_add',
                    'updatedAtAttribute' => false,
                    'value'              => new Expression('NOW()'),
                ],
                [
                    'class' => 'common\behaviors\ShowImage',
                ],
                'maps'       => [
                    'class'               => MapsBehavior::className(),
                    'location_attributes' => [
                        'city',
                        'street',
                        'house',
                    ],
                    'required_attributes' => [
                        'city',
                    ],
                ],
            ];
        }

        /**
         * @inheritdoc
         */
        public function rules()
        {
            return [
                [
                    [
                        'name',
                        'preview',
                        'city',
                        'cover',
                    ],
                    'required',
                ],
                [
                    [
                        'gallery_id',
                    ],
                    'integer',
                ],
                [
                    [
                        'description',
                        'cover',
                    ],
                    'string',
                ],
                [
                    [
                        'name',
                        'link',
                        'city',
                        'street',
                        'house',
                    ],
                    'string',
                    'max' => 255,
                ],
                [
                    [
                        'preview',
                    ],
                    'string',
                    'max' => 1000,
                ],
                [
                    [
                        'specializationInput',
                    ],
                    'safe',
                ],
            ];
        }

        /**
         * @inheritdoc
         */
        public function attributeLabels()
        {
            return [
                'portfolio_id'         => Yii::t('app', 'portfolio_id'),
                'user_id'              => Yii::t('app', 'user_id'),
                'name'                 => Yii::t('app', 'name'),
                'link'                 => Yii::t('app', 'link'),
                'date_add'             => Yii::t('app', 'date_add'),
                'user_add_id'          => Yii::t('app', 'user_add_id'),
                'view_count'           => Yii::t('app', 'view_count'),
                'city'                 => Yii::t('app', 'city'),
                'street'               => Yii::t('app', 'street'),
                'house'                => Yii::t('app', 'house'),
                'description'          => Yii::t('app', 'description'),
                'cover'                => Yii::t('app', 'cover_m'),
                'gallery_id'           => Yii::t('app', 'gallery_id'),
                'specializationString' => Yii::t('app', 'specializationString'),
                'preview'              => Yii::t('app', 'preview'),
            ];
        }

        /**
         * @return \yii\db\ActiveQuery
         */
        public function getPortfolioSpecializations()
        {
            return $this->hasMany(PortfolioSpecialization::className(), [ 'portfolio_id' => 'portfolio_id' ]);
        }

        public function getSpecializations()
        {
            return $this->hasMany(Specialization::className(), [ 'specialization_id' => 'specialization_id' ])
                        ->viaTable('portfolio_specialization', [ 'portfolio_id' => 'portfolio_id' ]);
        }

        public function getSpecializationInput()
        {
            return $this->getSpecializations()
                        ->asArray()
                        ->indexBy('specialization_id')
                        ->column();
        }

        public function setSpecializationInput($value)
        {
            $this->specializationInput = $value;
        }

        public function getSpecializationString()
        {
            return implode(', ', $this->getSpecializations()
                                      ->select([
                                          'specialization_name',
                                          'specialization_id',
                                      ])
                                      ->asArray()
                                      ->indexBy('specialization_id')
                                      ->column());
        }

        public function setSpecializationString($value)
        {
            $this->specializationString = $value;
        }

        public function getGallery()
        {
            return $this->hasOne(Gallery::className(), [ 'gallery_id' => 'gallery_id' ]);
        }

        /**
         * @return ActiveQuery
         */
        public function getPortfolioUsers()
        {
            return $this->hasMany(PortfolioUser::className(), [ 'portfolio_id' => 'portfolio_id' ])
                        ->with('user');
        }

        public function getUser()
        {
            return $this->hasOne(User::className(), [ 'id' => 'user_id' ]);
        }

        /**
         * @return ActiveQuery
         */
        public function getComments()
        {
            return $this->hasMany(Comment::className(), [ 'model_id' => 'portfolio_id' ])
                        ->andWhere([ 'comment.model'  => $this->className(),
                                     'comment.status' => Comment::STATUS_ACTIVE,
                        ]);
        }

        /**
         * @return ActiveQuery
         */
        public function getRating()
        {
            return $this->hasMany(Rating::className(), [ 'model_id' => 'comment_id' ])
                        ->via('comments')
                        ->andWhere([
                            'not',
                            [ 'rating.value' => NULL ],
                        ]);
        }

        /**
         * @return string
         */
        public function getRatingValue()
        {
            return $this->getRating()
                        ->asArray()
                        ->select([ 'sum' => 'ROUND(SUM(rating.value)/COUNT(rating.value)::numeric, 1)' ])
                        ->scalar();
        }
    }
